@@ -1,6 +1,9 @@
+from __future__ import annotations
+
 import os
 import sqlite3
 import subprocess
+from typing import Mapping
 
 import pytest
 from django.conf import settings
@@ -10,8 +13,8 @@ from django.utils.encoding import force_str
 # Construct names for the "inner" database used in runpytest tests
 _settings = settings.DATABASES["default"]
 
-DB_NAME = _settings["NAME"]
-TEST_DB_NAME = _settings["TEST"]["NAME"]
+DB_NAME: str = _settings["NAME"]
+TEST_DB_NAME: str = _settings["TEST"]["NAME"]
 
 if _settings["ENGINE"] == "django.db.backends.sqlite3" and TEST_DB_NAME is None:
     TEST_DB_NAME = ":memory:"
@@ -22,27 +25,28 @@ else:
 
     if TEST_DB_NAME is None:
         # No explicit test db name was given, construct a default one
-        TEST_DB_NAME = "test_{}_inner".format(DB_NAME)
+        TEST_DB_NAME = f"test_{DB_NAME}_inner"
     else:
         # An explicit test db name was given, is that as the base name
-        TEST_DB_NAME = "{}_inner".format(TEST_DB_NAME)
+        TEST_DB_NAME = f"{TEST_DB_NAME}_inner"
 
-    SECOND_DB_NAME = DB_NAME + '_second' if DB_NAME is not None else None
-    SECOND_TEST_DB_NAME = TEST_DB_NAME + '_second' if DB_NAME is not None else None
+    SECOND_DB_NAME = DB_NAME + "_second" if DB_NAME is not None else None
+    SECOND_TEST_DB_NAME = TEST_DB_NAME + "_second" if DB_NAME is not None else None
 
 
-def get_db_engine():
-    return _settings["ENGINE"].split(".")[-1]
+def get_db_engine() -> str:
+    db_engine: str = _settings["ENGINE"].split(".")[-1]
+    return db_engine
 
 
 class CmdResult:
-    def __init__(self, status_code, std_out, std_err):
+    def __init__(self, status_code: int, std_out: bytes, std_err: bytes) -> None:
         self.status_code = status_code
         self.std_out = std_out
         self.std_err = std_err
 
 
-def run_cmd(*args, env=None):
+def run_cmd(*args: str, env: Mapping[str, str] | None = None) -> CmdResult:
     r = subprocess.Popen(
         args,
         stdout=subprocess.PIPE,
@@ -54,7 +58,7 @@ def run_cmd(*args, env=None):
     return CmdResult(ret, stdoutdata, stderrdata)
 
 
-def run_psql(*args):
+def run_psql(*args: str) -> CmdResult:
     env = {}
     user = _settings.get("USER")
     if user:  # pragma: no branch
@@ -68,7 +72,7 @@ def run_psql(*args):
     return run_cmd("psql", *args, env=env)
 
 
-def run_mysql(*args):
+def run_mysql(*args: str) -> CmdResult:
     user = _settings.get("USER")
     if user:  # pragma: no branch
         args = ("-u", user, *args)
@@ -82,44 +86,39 @@ def run_mysql(*args):
     return run_cmd("mysql", *args)
 
 
-def skip_if_sqlite_in_memory():
-    if (
-        _settings["ENGINE"] == "django.db.backends.sqlite3"
-        and _settings["TEST"]["NAME"] is None
-    ):
+def skip_if_sqlite_in_memory() -> None:
+    if _settings["ENGINE"] == "django.db.backends.sqlite3" and _settings["TEST"]["NAME"] is None:
         pytest.skip("Do not test db reuse since database does not support it")
 
 
-def _get_db_name(db_suffix=None):
+def _get_db_name(db_suffix: str | None = None) -> str:
     name = TEST_DB_NAME
     if db_suffix:
-        name = "{}_{}".format(name, db_suffix)
+        name = f"{name}_{db_suffix}"
     return name
 
 
-def drop_database(db_suffix=None):
+def drop_database(db_suffix: str | None = None) -> None:
     name = _get_db_name(db_suffix)
     db_engine = get_db_engine()
 
     if db_engine == "postgresql":
-        r = run_psql("postgres", "-c", "DROP DATABASE %s" % name)
-        assert "DROP DATABASE" in force_str(
-            r.std_out
-        ) or "does not exist" in force_str(r.std_err)
+        r = run_psql("postgres", "-c", f"DROP DATABASE {name}")
+        assert "DROP DATABASE" in force_str(r.std_out) or "does not exist" in force_str(r.std_err)
         return
 
     if db_engine == "mysql":
-        r = run_mysql("-e", "DROP DATABASE %s" % name)
+        r = run_mysql("-e", f"DROP DATABASE {name}")
         assert "database doesn't exist" in force_str(r.std_err) or r.status_code == 0
         return
 
-    assert db_engine == "sqlite3", "%s cannot be tested properly!" % db_engine
+    assert db_engine == "sqlite3", f"{db_engine} cannot be tested properly!"
     assert name != ":memory:", "sqlite in-memory database cannot be dropped!"
     if os.path.exists(name):  # pragma: no branch
         os.unlink(name)
 
 
-def db_exists(db_suffix=None):
+def db_exists(db_suffix: str | None = None) -> bool:
     name = _get_db_name(db_suffix)
     db_engine = get_db_engine()
 
@@ -131,13 +130,12 @@ def db_exists(db_suffix=None):
         r = run_mysql(name, "-e", "SELECT 1")
         return r.status_code == 0
 
-    assert db_engine == "sqlite3", "%s cannot be tested properly!" % db_engine
-    assert TEST_DB_NAME != ":memory:", (
-        "sqlite in-memory database cannot be checked for existence!")
+    assert db_engine == "sqlite3", f"{db_engine} cannot be tested properly!"
+    assert TEST_DB_NAME != ":memory:", "sqlite in-memory database cannot be checked for existence!"
     return os.path.exists(name)
 
 
-def mark_database():
+def mark_database() -> None:
     db_engine = get_db_engine()
 
     if db_engine == "postgresql":
@@ -150,9 +148,8 @@ def mark_database():
         assert r.status_code == 0
         return
 
-    assert db_engine == "sqlite3", "%s cannot be tested properly!" % db_engine
-    assert TEST_DB_NAME != ":memory:", (
-        "sqlite in-memory database cannot be marked!")
+    assert db_engine == "sqlite3", f"{db_engine} cannot be tested properly!"
+    assert TEST_DB_NAME != ":memory:", "sqlite in-memory database cannot be marked!"
 
     conn = sqlite3.connect(TEST_DB_NAME)
     try:
@@ -162,7 +159,7 @@ def mark_database():
         conn.close()
 
 
-def mark_exists():
+def mark_exists() -> bool:
     db_engine = get_db_engine()
 
     if db_engine == "postgresql":
@@ -175,9 +172,8 @@ def mark_exists():
 
         return r.status_code == 0
 
-    assert db_engine == "sqlite3", "%s cannot be tested properly!" % db_engine
-    assert TEST_DB_NAME != ":memory:", (
-        "sqlite in-memory database cannot be checked for mark!")
+    assert db_engine == "sqlite3", f"{db_engine} cannot be tested properly!"
+    assert TEST_DB_NAME != ":memory:", "sqlite in-memory database cannot be checked for mark!"
 
     conn = sqlite3.connect(TEST_DB_NAME)
     try:
