@@ -28,6 +28,6 @@ TEMPLATES = [
     }
 ]
 
-DATABASE_ROUTERS = ['pytest_django_test.db_router.DbRouter']
+DATABASE_ROUTERS = ["pytest_django_test.db_router.DbRouter"]
 
 USE_TZ = True
