@@ -1,15 +1,6 @@
 from os import environ
 
-from .settings_base import *  # noqa: F401 F403
-
-
-# PyPy compatibility
-try:
-    from psycopg2cffi import compat
-
-    compat.register()
-except ImportError:
-    pass
+from .settings_base import *  # noqa: F403
 
 
 DATABASES = {
