@@ -1,6 +1,6 @@
 import tempfile
 
-from .settings_base import *  # noqa: F401 F403
+from .settings_base import *  # noqa: F403
 
 
 # This is a SQLite configuration, which uses a file based database for
