@@ -1,16 +1,21 @@
+from __future__ import annotations
+
+from typing import Generator
+
 import pytest
 from django.db import connection, transaction
 
-from pytest_django.lazy_django import get_django_version
+from .helpers import DjangoPytester
+
 from pytest_django_test.app.models import Item, SecondItem
 
 
 def db_supports_reset_sequences() -> bool:
     """Return if the current db engine supports `reset_sequences`."""
-    return (
-        connection.features.supports_transactions
-        and connection.features.supports_sequence_reset
+    ret: bool = (
+        connection.features.supports_transactions and connection.features.supports_sequence_reset
     )
+    return ret
 
 
 def test_noaccess() -> None:
@@ -48,23 +53,25 @@ def non_zero_sequences_counter(db: None) -> None:
 class TestDatabaseFixtures:
     """Tests for the different database fixtures."""
 
-    @pytest.fixture(params=[
-        "db",
-        "transactional_db",
-        "django_db_reset_sequences",
-        "django_db_serialized_rollback",
-    ])
-    def all_dbs(self, request) -> None:
+    @pytest.fixture(
+        params=[
+            "db",
+            "transactional_db",
+            "django_db_reset_sequences",
+            "django_db_serialized_rollback",
+        ]
+    )
+    def all_dbs(self, request: pytest.FixtureRequest) -> None:
         if request.param == "django_db_reset_sequences":
-            return request.getfixturevalue("django_db_reset_sequences")
+            request.getfixturevalue("django_db_reset_sequences")
         elif request.param == "transactional_db":
-            return request.getfixturevalue("transactional_db")
+            request.getfixturevalue("transactional_db")
         elif request.param == "db":
-            return request.getfixturevalue("db")
+            request.getfixturevalue("db")
         elif request.param == "django_db_serialized_rollback":
-            return request.getfixturevalue("django_db_serialized_rollback")
+            request.getfixturevalue("django_db_serialized_rollback")
         else:
-            assert False  # pragma: no cover
+            raise AssertionError()  # pragma: no cover
 
     def test_access(self, all_dbs: None) -> None:
         Item.objects.create(name="spam")
@@ -86,7 +93,8 @@ class TestDatabaseFixtures:
         assert not connection.in_atomic_block
 
     def test_transactions_enabled_via_reset_seq(
-        self, django_db_reset_sequences: None,
+        self,
+        django_db_reset_sequences: None,
     ) -> None:
         if not connection.features.supports_transactions:
             pytest.skip("transactions required for this test")
@@ -94,9 +102,11 @@ class TestDatabaseFixtures:
         assert not connection.in_atomic_block
 
     def test_django_db_reset_sequences_fixture(
-        self, db: None, django_testdir, non_zero_sequences_counter: None,
+        self,
+        db: None,
+        django_pytester: DjangoPytester,
+        non_zero_sequences_counter: None,
     ) -> None:
-
         if not db_supports_reset_sequences():
             pytest.skip(
                 "transactions and reset_sequences must be supported "
@@ -105,7 +115,7 @@ class TestDatabaseFixtures:
 
         # The test runs on a database that already contains objects, so its
         # id counter is > 1. We check for the ids of newly created objects.
-        django_testdir.create_test_module(
+        django_pytester.create_test_module(
             """
             import pytest
             from .app.models import Item
@@ -117,13 +127,11 @@ class TestDatabaseFixtures:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v", "--reuse-db")
-        result.stdout.fnmatch_lines(
-            ["*test_django_db_reset_sequences_requested PASSED*"]
-        )
+        result = django_pytester.runpytest_subprocess("-v", "--reuse-db")
+        result.stdout.fnmatch_lines(["*test_django_db_reset_sequences_requested PASSED*"])
 
-    def test_serialized_rollback(self, db: None, django_testdir) -> None:
-        django_testdir.create_app_file(
+    def test_serialized_rollback(self, db: None, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_app_file(
             """
             from django.db import migrations
 
@@ -143,7 +151,7 @@ class TestDatabaseFixtures:
             "migrations/0002_data_migration.py",
         )
 
-        django_testdir.create_test_module(
+        django_pytester.create_test_module(
             """
             import pytest
             from .app.models import Item
@@ -164,7 +172,7 @@ class TestDatabaseFixtures:
             """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
+        result = django_pytester.runpytest_subprocess("-v")
         assert result.ret == 0
 
     @pytest.fixture
@@ -186,15 +194,15 @@ class TestDatabaseFixtures:
         assert Item.objects.count() == 0
 
     @pytest.fixture
-    def fin(self, request, all_dbs: None) -> None:
+    def fin(self, request: pytest.FixtureRequest, all_dbs: None) -> Generator[None, None, None]:
         # This finalizer must be able to access the database
-        request.addfinalizer(lambda: Item.objects.create(name="spam"))
+        yield
+        Item.objects.create(name="spam")
 
     def test_fin(self, fin: None) -> None:
         # Check finalizer has db access (teardown will fail if not)
         pass
 
-    @pytest.mark.skipif(get_django_version() < (3, 2), reason="Django >= 3.2 required")
     def test_durable_transactions(self, all_dbs: None) -> None:
         with transaction.atomic(durable=True):
             item = Item.objects.create(name="foo")
@@ -240,7 +248,7 @@ class TestDatabaseFixturesAllOrder:
 
     # The test works when transactions are not supported, but it interacts
     # badly with other tests.
-    @pytest.mark.skipif('not connection.features.supports_transactions')
+    @pytest.mark.skipif("not connection.features.supports_transactions")
     def test_serialized_rollback(
         self,
         fixture_with_serialized_rollback: None,
@@ -283,78 +291,107 @@ class TestDatabaseMarker:
         assert not connection.in_atomic_block
 
     @pytest.mark.django_db
-    def test_reset_sequences_disabled(self, request) -> None:
+    def test_reset_sequences_disabled(self, request: pytest.FixtureRequest) -> None:
         marker = request.node.get_closest_marker("django_db")
         assert not marker.kwargs
 
     @pytest.mark.django_db(reset_sequences=True)
-    def test_reset_sequences_enabled(self, request) -> None:
+    def test_reset_sequences_enabled(self, request: pytest.FixtureRequest) -> None:
         marker = request.node.get_closest_marker("django_db")
         assert marker.kwargs["reset_sequences"]
 
     @pytest.mark.django_db(transaction=True, reset_sequences=True)
-    def test_transaction_reset_sequences_enabled(self, request) -> None:
+    def test_transaction_reset_sequences_enabled(self, request: pytest.FixtureRequest) -> None:
         marker = request.node.get_closest_marker("django_db")
         assert marker.kwargs["reset_sequences"]
 
-    @pytest.mark.django_db(databases=['default', 'replica', 'second'])
-    def test_databases(self, request) -> None:
+    @pytest.mark.django_db(databases=["default", "replica", "second"])
+    def test_databases(self, request: pytest.FixtureRequest) -> None:
         marker = request.node.get_closest_marker("django_db")
-        assert marker.kwargs["databases"] == ['default', 'replica', 'second']
+        assert marker.kwargs["databases"] == ["default", "replica", "second"]
 
-    @pytest.mark.django_db(databases=['second'])
-    def test_second_database(self, request) -> None:
+    @pytest.mark.django_db(databases=["second"])
+    def test_second_database(self, request: pytest.FixtureRequest) -> None:
         SecondItem.objects.create(name="spam")
 
-    @pytest.mark.django_db(databases=['default'])
-    def test_not_allowed_database(self, request) -> None:
-        with pytest.raises(AssertionError, match='not allowed'):
+    @pytest.mark.django_db(databases=["default"])
+    def test_not_allowed_database(self, request: pytest.FixtureRequest) -> None:
+        with pytest.raises(AssertionError, match="not allowed"):
             SecondItem.objects.count()
-        with pytest.raises(AssertionError, match='not allowed'):
+        with pytest.raises(AssertionError, match="not allowed"):
             SecondItem.objects.create(name="spam")
 
-    @pytest.mark.django_db(databases=['replica'])
-    def test_replica_database(self, request) -> None:
-        Item.objects.using('replica').count()
+    @pytest.mark.django_db(databases=["replica"])
+    def test_replica_database(self, request: pytest.FixtureRequest) -> None:
+        Item.objects.using("replica").count()
 
-    @pytest.mark.django_db(databases=['replica'])
-    def test_replica_database_not_allowed(self, request) -> None:
-        with pytest.raises(AssertionError, match='not allowed'):
+    @pytest.mark.django_db(databases=["replica"])
+    def test_replica_database_not_allowed(self, request: pytest.FixtureRequest) -> None:
+        with pytest.raises(AssertionError, match="not allowed"):
             Item.objects.count()
 
-    @pytest.mark.django_db(transaction=True, databases=['default', 'replica'])
-    def test_replica_mirrors_default_database(self, request) -> None:
-        Item.objects.create(name='spam')
-        Item.objects.using('replica').create(name='spam')
+    @pytest.mark.django_db(transaction=True, databases=["default", "replica"])
+    def test_replica_mirrors_default_database(self, request: pytest.FixtureRequest) -> None:
+        Item.objects.create(name="spam")
+        Item.objects.using("replica").create(name="spam")
 
         assert Item.objects.count() == 2
-        assert Item.objects.using('replica').count() == 2
+        assert Item.objects.using("replica").count() == 2
 
-    @pytest.mark.django_db(databases='__all__')
-    def test_all_databases(self, request) -> None:
+    @pytest.mark.django_db(databases="__all__")
+    def test_all_databases(self, request: pytest.FixtureRequest) -> None:
         Item.objects.count()
         Item.objects.create(name="spam")
         SecondItem.objects.count()
         SecondItem.objects.create(name="spam")
 
     @pytest.mark.django_db
-    def test_serialized_rollback_disabled(self, request):
+    def test_serialized_rollback_disabled(self, request: pytest.FixtureRequest):
         marker = request.node.get_closest_marker("django_db")
         assert not marker.kwargs
 
     # The test works when transactions are not supported, but it interacts
     # badly with other tests.
-    @pytest.mark.skipif('not connection.features.supports_transactions')
+    @pytest.mark.skipif("not connection.features.supports_transactions")
     @pytest.mark.django_db(serialized_rollback=True)
-    def test_serialized_rollback_enabled(self, request):
+    def test_serialized_rollback_enabled(self, request: pytest.FixtureRequest):
         marker = request.node.get_closest_marker("django_db")
         assert marker.kwargs["serialized_rollback"]
 
+    @pytest.mark.django_db
+    def test_available_apps_disabled(self, request: pytest.FixtureRequest) -> None:
+        marker = request.node.get_closest_marker("django_db")
+        assert not marker.kwargs
 
-def test_unittest_interaction(django_testdir) -> None:
+    @pytest.mark.django_db(available_apps=["pytest_django_test.app"])
+    def test_available_apps_enabled(self, request: pytest.FixtureRequest) -> None:
+        marker = request.node.get_closest_marker("django_db")
+        assert marker.kwargs["available_apps"] == ["pytest_django_test.app"]
+
+    @pytest.mark.django_db
+    def test_available_apps_default(self, request: pytest.FixtureRequest) -> None:
+        from django.apps import apps
+        from django.conf import settings
+
+        for app in settings.INSTALLED_APPS:
+            assert apps.is_installed(app)
+
+    @pytest.mark.django_db(available_apps=["pytest_django_test.app"])
+    def test_available_apps_limited(self, request: pytest.FixtureRequest) -> None:
+        from django.apps import apps
+        from django.conf import settings
+
+        assert apps.is_installed("pytest_django_test.app")
+
+        for app in settings.INSTALLED_APPS:
+            if app != "pytest_django_test.app":
+                assert not apps.is_installed(app)
+
+
+def test_unittest_interaction(django_pytester: DjangoPytester) -> None:
     "Test that (non-Django) unittests cannot access the DB."
 
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         """
         import pytest
         import unittest
@@ -382,7 +419,7 @@ def test_unittest_interaction(django_testdir) -> None:
     """
     )
 
-    result = django_testdir.runpytest_subprocess("-v", "--reuse-db")
+    result = django_pytester.runpytest_subprocess("-v", "--reuse-db")
     result.stdout.fnmatch_lines(
         [
             "*test_db_access_1 ERROR*",
@@ -395,18 +432,40 @@ def test_unittest_interaction(django_testdir) -> None:
     )
 
 
+def test_django_testcase_multi_db(django_pytester: DjangoPytester) -> None:
+    """Test that Django TestCase multi-db support works."""
+
+    django_pytester.create_test_module(
+        """
+        import pytest
+        from django.test import TestCase
+        from .app.models import Item, SecondItem
+
+        class TestCase(TestCase):
+            databases = ["default", "second"]
+
+            def test_db_access(self):
+                Item.objects.count() == 0
+                SecondItem.objects.count() == 0
+        """
+    )
+
+    result = django_pytester.runpytest_subprocess("-v", "--reuse-db")
+    result.assert_outcomes(passed=1)
+
+
 class Test_database_blocking:
-    def test_db_access_in_conftest(self, django_testdir) -> None:
+    def test_db_access_in_conftest(self, django_pytester: DjangoPytester) -> None:
         """Make sure database access in conftest module is prohibited."""
 
-        django_testdir.makeconftest(
+        django_pytester.makeconftest(
             """
             from tpkg.app.models import Item
             Item.objects.get()
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
+        result = django_pytester.runpytest_subprocess("-v")
         result.stderr.fnmatch_lines(
             [
                 '*RuntimeError: Database access not allowed, use the "django_db" mark, '
@@ -414,15 +473,15 @@ class Test_database_blocking:
             ]
         )
 
-    def test_db_access_in_test_module(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_db_access_in_test_module(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from tpkg.app.models import Item
             Item.objects.get()
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
+        result = django_pytester.runpytest_subprocess("-v")
         result.stdout.fnmatch_lines(
             [
                 '*RuntimeError: Database access not allowed, use the "django_db" mark, '
