@@ -1,4 +1,5 @@
 import os
+import sys
 
 import pytest
 from django.contrib.sites import models as site_models
@@ -7,12 +8,14 @@ from django.core import mail
 from django.db import connection
 from django.test import TestCase
 
+from .helpers import DjangoPytester
+
 from pytest_django_test.app.models import Item
 
 
 # It doesn't matter which order all the _again methods are run, we just need
 # to check the environment remains constant.
-# This is possible with some of the testdir magic, but this is the lazy way
+# This is possible with some of the pytester magic, but this is the lazy way
 # to do it.
 
 
@@ -51,8 +54,8 @@ class TestDirectAccessWorksForDjangoTestCase(TestCase):
     ROOT_URLCONF = 'tpkg.app.urls'
     """
 )
-def test_invalid_template_variable(django_testdir) -> None:
-    django_testdir.create_app_file(
+def test_invalid_template_variable(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_app_file(
         """
         from django.urls import path
 
@@ -62,7 +65,7 @@ def test_invalid_template_variable(django_testdir) -> None:
         """,
         "urls.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         """
         from django.shortcuts import render
 
@@ -72,13 +75,13 @@ def test_invalid_template_variable(django_testdir) -> None:
         """,
         "views.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         "<div>{{ invalid_var }}</div>", "templates/invalid_template_base.html"
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         "{% include 'invalid_template_base.html' %}", "templates/invalid_template.html"
     )
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         """
         import pytest
 
@@ -90,15 +93,98 @@ def test_invalid_template_variable(django_testdir) -> None:
             client.get('/invalid_template/')
         """
     )
-    result = django_testdir.runpytest_subprocess("-s", "--fail-on-template-vars")
+    result = django_pytester.runpytest_subprocess("-s", "--fail-on-template-vars")
 
     origin = "'*/tpkg/app/templates/invalid_template_base.html'"
     result.stdout.fnmatch_lines_random(
         [
             "tpkg/test_the_test.py F.*",
-            "E * Failed: Undefined template variable 'invalid_var' in {}".format(
-                origin
-            ),
+            f"E * Failed: Undefined template variable 'invalid_var' in {origin}",
+        ]
+    )
+
+
+@pytest.mark.django_project(
+    extra_settings="""
+    TEMPLATE_LOADERS = (
+        'django.template.loaders.filesystem.Loader',
+        'django.template.loaders.app_directories.Loader',
+    )
+    """
+)
+def test_invalid_template_variable_marker_cleanup(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_app_file(
+        "<div>{{ invalid_var }}</div>", "templates/invalid_template_base.html"
+    )
+    django_pytester.create_app_file(
+        "{% include 'invalid_template_base.html' %}", "templates/invalid_template.html"
+    )
+    django_pytester.create_test_module(
+        """
+        from django.template.loader import render_to_string
+
+        import pytest
+
+        @pytest.mark.ignore_template_errors
+        def test_ignore(client):
+            render_to_string('invalid_template.html')
+
+        def test_for_invalid_template(client):
+            render_to_string('invalid_template.html')
+
+        """
+    )
+    result = django_pytester.runpytest_subprocess("-s", "--fail-on-template-vars")
+
+    origin = "'*/tpkg/app/templates/invalid_template_base.html'"
+    result.stdout.fnmatch_lines_random(
+        [
+            "tpkg/test_the_test.py .F*",
+            f"E * Failed: Undefined template variable 'invalid_var' in {origin}",
+        ]
+    )
+
+
+@pytest.mark.django_project(
+    extra_settings="""
+    TEMPLATE_LOADERS = (
+        'django.template.loaders.filesystem.Loader',
+        'django.template.loaders.app_directories.Loader',
+    )
+    TEMPLATES[0]["OPTIONS"]["string_if_invalid"] = "Something clever"
+    """
+)
+def test_invalid_template_variable_behaves_normally_when_ignored(
+    django_pytester: DjangoPytester,
+) -> None:
+    django_pytester.create_app_file(
+        "<div>{{ invalid_var }}</div>", "templates/invalid_template_base.html"
+    )
+    django_pytester.create_app_file(
+        "{% include 'invalid_template_base.html' %}", "templates/invalid_template.html"
+    )
+    django_pytester.create_test_module(
+        """
+        from django.template.loader import render_to_string
+
+        import pytest
+
+        @pytest.mark.ignore_template_errors
+        def test_ignore(client):
+            assert render_to_string('invalid_template.html') == "<div>Something clever</div>"
+
+        def test_for_invalid_template(client):
+            render_to_string('invalid_template.html')
+
+        """
+    )
+    result = django_pytester.runpytest_subprocess("-s", "--fail-on-template-vars")
+
+    origin = "'*/tpkg/app/templates/invalid_template_base.html'"
+    result.stdout.fnmatch_lines_random(
+        [
+            "tpkg/test_the_test.py .F*",
+            f"E * Failed: Undefined template variable 'invalid_var' in {origin}",
         ]
     )
 
@@ -112,8 +198,8 @@ def test_invalid_template_variable(django_testdir) -> None:
     ROOT_URLCONF = 'tpkg.app.urls'
     """
 )
-def test_invalid_template_with_default_if_none(django_testdir) -> None:
-    django_testdir.create_app_file(
+def test_invalid_template_with_default_if_none(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_app_file(
         """
             <div>{{ data.empty|default:'d' }}</div>
             <div>{{ data.none|default:'d' }}</div>
@@ -123,7 +209,7 @@ def test_invalid_template_with_default_if_none(django_testdir) -> None:
         """,
         "templates/the_template.html",
     )
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         """
         def test_for_invalid_template():
             from django.shortcuts import render
@@ -136,7 +222,7 @@ def test_invalid_template_with_default_if_none(django_testdir) -> None:
             )
         """
     )
-    result = django_testdir.runpytest_subprocess("--fail-on-template-vars")
+    result = django_pytester.runpytest_subprocess("--fail-on-template-vars")
     result.stdout.fnmatch_lines(
         [
             "tpkg/test_the_test.py F",
@@ -154,8 +240,8 @@ def test_invalid_template_with_default_if_none(django_testdir) -> None:
     ROOT_URLCONF = 'tpkg.app.urls'
     """
 )
-def test_invalid_template_variable_opt_in(django_testdir) -> None:
-    django_testdir.create_app_file(
+def test_invalid_template_variable_opt_in(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_app_file(
         """
         from django.urls import path
 
@@ -165,7 +251,7 @@ def test_invalid_template_variable_opt_in(django_testdir) -> None:
         """,
         "urls.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         """
         from django.shortcuts import render
 
@@ -175,10 +261,10 @@ def test_invalid_template_variable_opt_in(django_testdir) -> None:
         """,
         "views.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         "<div>{{ invalid_var }}</div>", "templates/invalid_template.html"
     )
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         """
         import pytest
 
@@ -190,7 +276,7 @@ def test_invalid_template_variable_opt_in(django_testdir) -> None:
             client.get('/invalid_template/')
         """
     )
-    result = django_testdir.runpytest_subprocess("-s")
+    result = django_pytester.runpytest_subprocess("-s")
     result.stdout.fnmatch_lines_random(["tpkg/test_the_test.py ..*"])
 
 
@@ -222,9 +308,8 @@ class TestrunnerVerbosity:
     pytest's verbosity level."""
 
     @pytest.fixture
-    def testdir(self, django_testdir):
-        print("testdir")
-        django_testdir.create_test_module(
+    def pytester(self, django_pytester: DjangoPytester) -> pytest.Pytester:
+        django_pytester.create_test_module(
             """
             import pytest
 
@@ -233,29 +318,27 @@ class TestrunnerVerbosity:
                 pass
             """
         )
-        return django_testdir
+        return django_pytester
 
-    def test_default(self, testdir) -> None:
+    def test_default(self, pytester: pytest.Pytester) -> None:
         """Not verbose by default."""
-        result = testdir.runpytest_subprocess("-s")
+        result = pytester.runpytest_subprocess("-s")
         result.stdout.fnmatch_lines(["tpkg/test_the_test.py .*"])
 
-    def test_vq_verbosity_0(self, testdir) -> None:
+    def test_vq_verbosity_0(self, pytester: pytest.Pytester) -> None:
         """-v and -q results in verbosity 0."""
-        result = testdir.runpytest_subprocess("-s", "-v", "-q")
+        result = pytester.runpytest_subprocess("-s", "-v", "-q")
         result.stdout.fnmatch_lines(["tpkg/test_the_test.py .*"])
 
-    def test_verbose_with_v(self, testdir) -> None:
+    def test_verbose_with_v(self, pytester: pytest.Pytester) -> None:
         """Verbose output with '-v'."""
-        result = testdir.runpytest_subprocess("-s", "-v")
+        result = pytester.runpytest_subprocess("-s", "-v")
         result.stdout.fnmatch_lines_random(["tpkg/test_the_test.py:*", "*PASSED*"])
-        result.stderr.fnmatch_lines(
-            ["*Destroying test database for alias 'default'*"]
-        )
+        result.stderr.fnmatch_lines(["*Destroying test database for alias 'default'*"])
 
-    def test_more_verbose_with_vv(self, testdir) -> None:
+    def test_more_verbose_with_vv(self, pytester: pytest.Pytester) -> None:
         """More verbose output with '-v -v'."""
-        result = testdir.runpytest_subprocess("-s", "-v", "-v")
+        result = pytester.runpytest_subprocess("-s", "-v", "-v")
         result.stdout.fnmatch_lines_random(
             [
                 "tpkg/test_the_test.py:*",
@@ -271,20 +354,17 @@ class TestrunnerVerbosity:
             ]
         )
 
-    def test_more_verbose_with_vv_and_reusedb(self, testdir) -> None:
+    def test_more_verbose_with_vv_and_reusedb(self, pytester: pytest.Pytester) -> None:
         """More verbose output with '-v -v', and --create-db."""
-        result = testdir.runpytest_subprocess("-s", "-v", "-v", "--create-db")
+        result = pytester.runpytest_subprocess("-s", "-v", "-v", "--create-db")
         result.stdout.fnmatch_lines(["tpkg/test_the_test.py:*", "*PASSED*"])
         result.stderr.fnmatch_lines(["*Creating test database for alias*"])
-        assert (
-            "*Destroying test database for alias 'default' ('*')...*"
-            not in result.stderr.str()
-        )
+        assert "*Destroying test database for alias 'default' ('*')...*" not in result.stderr.str()
 
 
 @pytest.mark.django_db
 @pytest.mark.parametrize("site_name", ["site1", "site2"])
-def test_clear_site_cache(site_name: str, rf, monkeypatch) -> None:
+def test_clear_site_cache(site_name: str, rf, monkeypatch: pytest.MonkeyPatch) -> None:
     request = rf.get("/")
     monkeypatch.setattr(request, "get_host", lambda: "foo.com")
     Site.objects.create(domain="foo.com", name=site_name)
@@ -299,3 +379,42 @@ def test_clear_site_cache_check_site_cache_size(site_name: str, settings) -> Non
     settings.SITE_ID = site.id
     assert Site.objects.get_current() == site
     assert len(site_models.SITE_CACHE) == 1
+
+
+@pytest.mark.django_project(
+    project_root="django_project_root",
+    create_manage_py=True,
+    extra_settings="""
+    TEST_RUNNER = 'pytest_django.runner.TestRunner'
+    """,
+)
+def test_manage_test_runner(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
+        """
+        import pytest
+
+        @pytest.mark.django_db
+        def test_inner_testrunner():
+            pass
+        """
+    )
+    result = django_pytester.run(*[sys.executable, "django_project_root/manage.py", "test"])
+    assert "1 passed" in "\n".join(result.outlines)
+
+
+@pytest.mark.django_project(
+    project_root="django_project_root",
+    create_manage_py=True,
+)
+def test_manage_test_runner_without(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
+        """
+        import pytest
+
+        @pytest.mark.django_db
+        def test_inner_testrunner():
+            pass
+        """
+    )
+    result = django_pytester.run(*[sys.executable, "django_project_root/manage.py", "test"])
+    assert "Found 0 test(s)." in "\n".join(result.outlines)
