@@ -3,6 +3,7 @@
 Not quite all fixtures are tested here, the db and transactional_db
 fixtures are tested in test_database.
 """
+
 import socket
 from contextlib import contextmanager
 from typing import Generator
@@ -13,15 +14,18 @@ import pytest
 from django.conf import settings as real_settings
 from django.core import mail
 from django.db import connection, transaction
-from django.test.client import Client, RequestFactory
+from django.test import AsyncClient, AsyncRequestFactory, Client, RequestFactory
+from django.utils.connection import ConnectionDoesNotExist
 from django.utils.encoding import force_str
 
-from pytest_django.lazy_django import get_django_version
+from .helpers import DjangoPytester
+
+from pytest_django import DjangoAssertNumQueries, DjangoCaptureOnCommitCallbacks, DjangoDbBlocker
 from pytest_django_test.app.models import Item
 
 
 @contextmanager
-def nonverbose_config(config) -> Generator[None, None, None]:
+def nonverbose_config(config: pytest.Config) -> Generator[None, None, None]:
     """Ensure that pytest's config.option.verbose is <= 0."""
     if config.option.verbose <= 0:
         yield
@@ -32,14 +36,11 @@ def nonverbose_config(config) -> Generator[None, None, None]:
         config.option.verbose = saved
 
 
-def test_client(client) -> None:
+def test_client(client: Client) -> None:
     assert isinstance(client, Client)
 
 
-@pytest.mark.skipif(get_django_version() < (3, 1), reason="Django >= 3.1 required")
-def test_async_client(async_client) -> None:
-    from django.test.client import AsyncClient
-
+def test_async_client(async_client: AsyncClient) -> None:
     assert isinstance(async_client, AsyncClient)
 
 
@@ -59,7 +60,7 @@ def test_admin_client_no_db_marker(admin_client: Client) -> None:
 # For test below.
 @pytest.fixture
 def existing_admin_user(django_user_model):
-    return django_user_model._default_manager.create_superuser('admin', None, None)
+    return django_user_model._default_manager.create_superuser("admin", None, None)
 
 
 def test_admin_client_existing_user(
@@ -81,19 +82,19 @@ def test_admin_user_no_db_marker(admin_user, django_user_model) -> None:
     assert isinstance(admin_user, django_user_model)
 
 
-def test_rf(rf) -> None:
+def test_rf(rf: RequestFactory) -> None:
     assert isinstance(rf, RequestFactory)
 
 
-@pytest.mark.skipif(get_django_version() < (3, 1), reason="Django >= 3.1 required")
-def test_async_rf(async_rf) -> None:
-    from django.test.client import AsyncRequestFactory
-
+def test_async_rf(async_rf: AsyncRequestFactory) -> None:
     assert isinstance(async_rf, AsyncRequestFactory)
 
 
 @pytest.mark.django_db
-def test_django_assert_num_queries_db(request, django_assert_num_queries) -> None:
+def test_django_assert_num_queries_db(
+    request: pytest.FixtureRequest,
+    django_assert_num_queries: DjangoAssertNumQueries,
+) -> None:
     with nonverbose_config(request.config):
         with django_assert_num_queries(3):
             Item.objects.create(name="foo")
@@ -104,20 +105,22 @@ def test_django_assert_num_queries_db(request, django_assert_num_queries) -> Non
             with django_assert_num_queries(2) as captured:
                 Item.objects.create(name="quux")
         assert excinfo.value.args == (
-            "Expected to perform 2 queries but 1 was done "
-            "(add -v option to show queries)",
+            "Expected to perform 2 queries but 1 was done (add -v option to show queries)",
         )
         assert len(captured.captured_queries) == 1
 
 
 @pytest.mark.django_db
-def test_django_assert_max_num_queries_db(request, django_assert_max_num_queries) -> None:
+def test_django_assert_max_num_queries_db(
+    request: pytest.FixtureRequest,
+    django_assert_max_num_queries: DjangoAssertNumQueries,
+) -> None:
     with nonverbose_config(request.config):
         with django_assert_max_num_queries(2):
             Item.objects.create(name="1-foo")
             Item.objects.create(name="2-bar")
 
-        with pytest.raises(pytest.fail.Exception) as excinfo:
+        with pytest.raises(pytest.fail.Exception) as excinfo:  # noqa: PT012
             with django_assert_max_num_queries(2) as captured:
                 Item.objects.create(name="1-foo")
                 Item.objects.create(name="2-bar")
@@ -133,7 +136,9 @@ def test_django_assert_max_num_queries_db(request, django_assert_max_num_queries
 
 @pytest.mark.django_db(transaction=True)
 def test_django_assert_num_queries_transactional_db(
-    request, transactional_db: None, django_assert_num_queries
+    request: pytest.FixtureRequest,
+    transactional_db: None,
+    django_assert_num_queries: DjangoAssertNumQueries,
 ) -> None:
     with nonverbose_config(request.config):
         with transaction.atomic():
@@ -147,8 +152,8 @@ def test_django_assert_num_queries_transactional_db(
                     Item.objects.create(name="quux")
 
 
-def test_django_assert_num_queries_output(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_django_assert_num_queries_output(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.contrib.contenttypes.models import ContentType
         import pytest
@@ -160,13 +165,13 @@ def test_django_assert_num_queries_output(django_testdir) -> None:
                 ContentType.objects.count()
     """
     )
-    result = django_testdir.runpytest_subprocess("--tb=short")
+    result = django_pytester.runpytest_subprocess("--tb=short")
     result.stdout.fnmatch_lines(["*Expected to perform 1 queries but 2 were done*"])
     assert result.ret == 1
 
 
-def test_django_assert_num_queries_output_verbose(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_django_assert_num_queries_output_verbose(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.contrib.contenttypes.models import ContentType
         import pytest
@@ -178,7 +183,7 @@ def test_django_assert_num_queries_output_verbose(django_testdir) -> None:
                 ContentType.objects.count()
     """
     )
-    result = django_testdir.runpytest_subprocess("--tb=short", "-v")
+    result = django_pytester.runpytest_subprocess("--tb=short", "-v")
     result.stdout.fnmatch_lines(
         ["*Expected to perform 11 queries but 2 were done*", "*Queries:*", "*========*"]
     )
@@ -186,7 +191,9 @@ def test_django_assert_num_queries_output_verbose(django_testdir) -> None:
 
 
 @pytest.mark.django_db
-def test_django_assert_num_queries_db_connection(django_assert_num_queries) -> None:
+def test_django_assert_num_queries_db_connection(
+    django_assert_num_queries: DjangoAssertNumQueries,
+) -> None:
     from django.db import connection
 
     with django_assert_num_queries(1, connection=connection):
@@ -201,8 +208,30 @@ def test_django_assert_num_queries_db_connection(django_assert_num_queries) -> N
 
 
 @pytest.mark.django_db
-def test_django_assert_num_queries_output_info(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_django_assert_num_queries_db_using(
+    django_assert_num_queries: DjangoAssertNumQueries,
+) -> None:
+    from django.db import connection
+
+    with django_assert_num_queries(1, using="default"):
+        Item.objects.create(name="foo")
+
+    error_message = 'The "connection" and "using" parameter cannot be used together'
+    with pytest.raises(ValueError, match=error_message):
+        with django_assert_num_queries(1, connection=connection, using="default"):
+            Item.objects.create(name="foo")
+
+    with django_assert_num_queries(1, using=None):
+        Item.objects.create(name="foo")
+
+    with pytest.raises(ConnectionDoesNotExist):
+        with django_assert_num_queries(1, using="bad_db_name"):
+            pass
+
+
+@pytest.mark.django_db
+def test_django_assert_num_queries_output_info(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.contrib.contenttypes.models import ContentType
         import pytest
@@ -218,7 +247,7 @@ def test_django_assert_num_queries_output_info(django_testdir) -> None:
                 ContentType.objects.first()  # additional wrong query
     """
     )
-    result = django_testdir.runpytest_subprocess("--tb=short", "-v")
+    result = django_pytester.runpytest_subprocess("--tb=short", "-v")
     result.stdout.fnmatch_lines(
         [
             "*Expected to perform 2 queries but 3 were done*",
@@ -231,7 +260,9 @@ def test_django_assert_num_queries_output_info(django_testdir) -> None:
 
 
 @pytest.mark.django_db
-def test_django_capture_on_commit_callbacks(django_capture_on_commit_callbacks) -> None:
+def test_django_capture_on_commit_callbacks(
+    django_capture_on_commit_callbacks: DjangoCaptureOnCommitCallbacks,
+) -> None:
     if not connection.features.supports_transactions:
         pytest.skip("transactions required for this test")
 
@@ -254,7 +285,9 @@ def test_django_capture_on_commit_callbacks(django_capture_on_commit_callbacks)
 
 
 @pytest.mark.django_db(databases=["default", "second"])
-def test_django_capture_on_commit_callbacks_multidb(django_capture_on_commit_callbacks) -> None:
+def test_django_capture_on_commit_callbacks_multidb(
+    django_capture_on_commit_callbacks: DjangoCaptureOnCommitCallbacks,
+) -> None:
     if not connection.features.supports_transactions:
         pytest.skip("transactions required for this test")
 
@@ -281,7 +314,7 @@ def test_django_capture_on_commit_callbacks_multidb(django_capture_on_commit_cal
 
 @pytest.mark.django_db(transaction=True)
 def test_django_capture_on_commit_callbacks_transactional(
-    django_capture_on_commit_callbacks,
+    django_capture_on_commit_callbacks: DjangoCaptureOnCommitCallbacks,
 ) -> None:
     if not connection.features.supports_transactions:
         pytest.skip("transactions required for this test")
@@ -333,7 +366,7 @@ class TestSettings:
     def test_signals(self, settings) -> None:
         result = []
 
-        def assert_signal(signal, sender, setting, value, enter):
+        def assert_signal(signal, sender, setting, value, enter) -> None:
             result.append((setting, value, enter))
 
         from django.test.signals import setting_changed
@@ -352,8 +385,8 @@ class TestSettings:
         settings.FOOBAR = "abc123"
         assert sorted(result) == [("FOOBAR", "abc123", True)]
 
-    def test_modification_signal(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_modification_signal(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             import pytest
 
@@ -386,7 +419,7 @@ class TestSettings:
          """
         )
 
-        result = django_testdir.runpytest_subprocess("--tb=short", "-v", "-s")
+        result = django_pytester.runpytest_subprocess("--tb=short", "-v", "-s")
 
         # test_set
         result.stdout.fnmatch_lines(
@@ -414,7 +447,7 @@ class TestLiveServer:
         from django.conf import settings
 
         assert (
-            "{}.{}".format(settings.__class__.__module__, settings.__class__.__name__)
+            f"{settings.__class__.__module__}.{settings.__class__.__name__}"
             == "django.conf.Settings"
         )
         TestLiveServer._test_settings_before_run = True  # type: ignore[attr-defined]
@@ -431,7 +464,7 @@ class TestLiveServer:
 
         assert TestLiveServer._test_settings_before_run is True  # type: ignore[attr-defined]
         assert (
-            "{}.{}".format(settings.__class__.__module__, settings.__class__.__name__)
+            f"{settings.__class__.__module__}.{settings.__class__.__name__}"
             == "django.conf.Settings"
         )
         assert settings.ALLOWED_HOSTS == ["testserver"]
@@ -460,17 +493,19 @@ class TestLiveServer:
         assert force_str(response_data) == "Item count: 1"
 
     @pytest.fixture
-    def item(self) -> None:
+    def item(self) -> Item:
         # This has not requested database access explicitly, but the
         # live_server fixture auto-uses the transactional_db fixture.
-        Item.objects.create(name="foo")
+        item: Item = Item.objects.create(name="foo")
+        return item
 
-    def test_item(self, item, live_server) -> None:
+    def test_item(self, item: Item, live_server) -> None:
         pass
 
     @pytest.fixture
     def item_db(self, db: None) -> Item:
-        return Item.objects.create(name="foo")
+        item: Item = Item.objects.create(name="foo")
+        return item
 
     def test_item_db(self, item_db: Item, live_server) -> None:
         response_data = urlopen(live_server + "/item_count/").read()
@@ -478,7 +513,8 @@ class TestLiveServer:
 
     @pytest.fixture
     def item_transactional_db(self, transactional_db: None) -> Item:
-        return Item.objects.create(name="foo")
+        item: Item = Item.objects.create(name="foo")
+        return item
 
     def test_item_transactional_db(self, item_transactional_db: Item, live_server) -> None:
         response_data = urlopen(live_server + "/item_count/").read()
@@ -498,12 +534,16 @@ class TestLiveServer:
         STATIC_URL = '/static/'
         """
     )
-    def test_serve_static_with_staticfiles_app(self, django_testdir, settings) -> None:
+    def test_serve_static_with_staticfiles_app(
+        self,
+        django_pytester: DjangoPytester,
+        settings,
+    ) -> None:
         """
         LiveServer always serves statics with ``django.contrib.staticfiles``
         handler.
         """
-        django_testdir.create_test_module(
+        django_pytester.create_test_module(
             """
             from urllib.request import urlopen
 
@@ -518,7 +558,7 @@ class TestLiveServer:
                     assert force_str(response_data) == 'bla\\n'
             """
         )
-        result = django_testdir.runpytest_subprocess("--tb=short", "-v")
+        result = django_pytester.runpytest_subprocess("--tb=short", "-v")
         result.stdout.fnmatch_lines(["*test_a*PASSED*"])
         assert result.ret == 0
 
@@ -530,7 +570,7 @@ class TestLiveServer:
         with pytest.raises(HTTPError):
             urlopen(live_server + "/static/a_file.txt").read()
 
-    def test_specified_port_django_111(self, django_testdir) -> None:
+    def test_specified_port_django_111(self, django_pytester: DjangoPytester) -> None:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         try:
             sock.bind(("", 0))
@@ -538,18 +578,17 @@ class TestLiveServer:
         finally:
             sock.close()
 
-        django_testdir.create_test_module(
+        django_pytester.create_test_module(
+            f"""
+            def test_with_live_server(live_server):
+                assert live_server.port == {port}
             """
-        def test_with_live_server(live_server):
-            assert live_server.port == %d
-        """
-            % port
         )
 
-        django_testdir.runpytest_subprocess("--liveserver=localhost:%s" % port)
+        django_pytester.runpytest_subprocess(f"--liveserver=localhost:{port}")
 
 
-@pytest.mark.parametrize("username_field", ("email", "identifier"))
+@pytest.mark.parametrize("username_field", ["email", "identifier"])
 @pytest.mark.django_project(
     extra_settings="""
     AUTH_USER_MODEL = 'app.MyCustomUser'
@@ -563,9 +602,9 @@ class TestLiveServer:
     ROOT_URLCONF = 'tpkg.app.urls'
     """
 )
-def test_custom_user_model(django_testdir, username_field) -> None:
-    django_testdir.create_app_file(
-        """
+def test_custom_user_model(django_pytester: DjangoPytester, username_field: str) -> None:
+    django_pytester.create_app_file(
+        f"""
         from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
         from django.db import models
 
@@ -599,10 +638,10 @@ def test_custom_user_model(django_testdir, username_field) -> None:
             objects = MyCustomUserManager()
 
             USERNAME_FIELD = '{username_field}'
-        """.format(username_field=username_field),
+        """,
         "models.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         """
         from django.urls import path
 
@@ -612,7 +651,7 @@ def test_custom_user_model(django_testdir, username_field) -> None:
         """,
         "urls.py",
     )
-    django_testdir.create_app_file(
+    django_pytester.create_app_file(
         """
         from django.http import HttpResponse
         from django.template import Template
@@ -625,7 +664,7 @@ def test_custom_user_model(django_testdir, username_field) -> None:
         """,
         "views.py",
     )
-    django_testdir.makepyfile(
+    django_pytester.makepyfile(
         """
         from django.utils.encoding import force_str
         from tpkg.app.models import MyCustomUser
@@ -636,8 +675,8 @@ def test_custom_user_model(django_testdir, username_field) -> None:
         """
     )
 
-    django_testdir.create_app_file("", "migrations/__init__.py")
-    django_testdir.create_app_file(
+    django_pytester.create_app_file("", "migrations/__init__.py")
+    django_pytester.create_app_file(
         """
 from django.db import models, migrations
 import django.utils.timezone
@@ -672,18 +711,18 @@ class Migration(migrations.Migration):
             bases=None,
         ),
     ]
-        """,  # noqa: E501
+        """,
         "migrations/0002_custom_user_model.py",
     )
 
-    result = django_testdir.runpytest_subprocess("-s")
+    result = django_pytester.runpytest_subprocess("-s")
     result.stdout.fnmatch_lines(["* 1 passed*"])
     assert result.ret == 0
 
 
 class Test_django_db_blocker:
     @pytest.mark.django_db
-    def test_block_manually(self, django_db_blocker) -> None:
+    def test_block_manually(self, django_db_blocker: DjangoDbBlocker) -> None:
         try:
             django_db_blocker.block()
             with pytest.raises(RuntimeError):
@@ -692,27 +731,25 @@ class Test_django_db_blocker:
             django_db_blocker.restore()
 
     @pytest.mark.django_db
-    def test_block_with_block(self, django_db_blocker) -> None:
+    def test_block_with_block(self, django_db_blocker: DjangoDbBlocker) -> None:
         with django_db_blocker.block():
             with pytest.raises(RuntimeError):
                 Item.objects.exists()
 
-    def test_unblock_manually(self, django_db_blocker) -> None:
+    def test_unblock_manually(self, django_db_blocker: DjangoDbBlocker) -> None:
         try:
             django_db_blocker.unblock()
             Item.objects.exists()
         finally:
             django_db_blocker.restore()
 
-    def test_unblock_with_block(self, django_db_blocker) -> None:
+    def test_unblock_with_block(self, django_db_blocker: DjangoDbBlocker) -> None:
         with django_db_blocker.unblock():
             Item.objects.exists()
 
 
 def test_mail(mailoutbox) -> None:
-    assert (
-        mailoutbox is mail.outbox
-    )  # check that mail.outbox and fixture value is same object
+    assert mailoutbox is mail.outbox  # check that mail.outbox and fixture value is same object
     assert len(mailoutbox) == 0
     mail.send_mail("subject", "body", "from@example.com", ["to@example.com"])
     assert len(mailoutbox) == 1
@@ -734,8 +771,8 @@ def test_mail_message_uses_mocked_DNS_NAME(mailoutbox) -> None:
     assert message["Message-ID"].endswith("@fake-tests.example.com>")
 
 
-def test_mail_message_uses_django_mail_dnsname_fixture(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_mail_message_uses_django_mail_dnsname_fixture(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.core import mail
         import pytest
@@ -752,13 +789,13 @@ def test_mail_message_uses_django_mail_dnsname_fixture(django_testdir) -> None:
             assert message['Message-ID'].endswith('@from.django_mail_dnsname>')
     """
     )
-    result = django_testdir.runpytest_subprocess("--tb=short", "-v")
+    result = django_pytester.runpytest_subprocess("--tb=short", "-v")
     result.stdout.fnmatch_lines(["*test_mailbox_inner*PASSED*"])
     assert result.ret == 0
 
 
-def test_mail_message_dns_patching_can_be_skipped(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_mail_message_dns_patching_can_be_skipped(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.core import mail
         import pytest
@@ -785,8 +822,6 @@ def test_mail_message_dns_patching_can_be_skipped(django_testdir) -> None:
             assert mocked_make_msgid.called[0][1]['domain'] is mail.DNS_NAME
     """
     )
-    result = django_testdir.runpytest_subprocess("--tb=short", "-vv", "-s")
-    result.stdout.fnmatch_lines(
-        ["*test_mailbox_inner*", "django_mail_dnsname_mark", "PASSED*"]
-    )
+    result = django_pytester.runpytest_subprocess("--tb=short", "-vv", "-s")
+    result.stdout.fnmatch_lines(["*test_mailbox_inner*", "django_mail_dnsname_mark", "PASSED*"])
     assert result.ret == 0
