@@ -1,11 +1,13 @@
 import pytest
-from django.test import TestCase
+from django.test import SimpleTestCase, TestCase, tag
+
+from .helpers import DjangoPytester
 
 from pytest_django_test.app.models import Item
 
 
 class TestFixtures(TestCase):
-    fixtures = ["items"]
+    fixtures = ("items",)
 
     def test_fixtures(self) -> None:
         assert Item.objects.count() == 1
@@ -24,10 +26,10 @@ class TestSetup(TestCase):
         Item.objects.create(name="Some item again")
 
     def test_count(self) -> None:
-        self.assertEqual(Item.objects.count(), 2)
+        self.assertEqual(Item.objects.count(), 2)  # noqa: PT009
         assert Item.objects.count() == 2
         Item.objects.create(name="Foo")
-        self.assertEqual(Item.objects.count(), 3)
+        self.assertEqual(Item.objects.count(), 3)  # noqa: PT009
 
     def test_count_again(self) -> None:
         self.test_count()
@@ -38,7 +40,7 @@ class TestSetup(TestCase):
 
 
 class TestFixturesWithSetup(TestCase):
-    fixtures = ["items"]
+    fixtures = ("items",)
 
     def setUp(self) -> None:
         assert Item.objects.count() == 1
@@ -55,14 +57,49 @@ class TestFixturesWithSetup(TestCase):
         assert Item.objects.count() == 3
 
 
-def test_sole_test(django_testdir) -> None:
+@tag("tag1", "tag2")
+class TestDjangoTagsToPytestMarkers(SimpleTestCase):
+    """Django test tags are converted to Pytest markers, at the class & method
+    levels."""
+
+    @pytest.fixture(autouse=True)
+    def gimme_my_markers(self, request: pytest.FixtureRequest) -> None:
+        self.markers = {m.name for m in request.node.iter_markers()}
+
+    @tag("tag3", "tag4")  # type: ignore[misc]
+    def test_1(self) -> None:
+        assert self.markers == {"tag1", "tag2", "tag3", "tag4"}
+
+    def test_2(self) -> None:
+        assert self.markers == {"tag1", "tag2"}
+
+    @tag("tag5")  # type: ignore[misc]
+    def test_3(self) -> None:
+        assert self.markers == {"tag1", "tag2", "tag5"}
+
+
+@tag("tag1")
+class TestNonDjangoClassWithTags:
+    """Django test tags are only converted to Pytest markers if actually
+    Django tests. Use pytest markers directly for pytest tests."""
+
+    @pytest.fixture(autouse=True)
+    def gimme_my_markers(self, request: pytest.FixtureRequest) -> None:
+        self.markers = {m.name for m in request.node.iter_markers()}
+
+    @tag("tag2")  # type: ignore[misc]
+    def test_1(self) -> None:
+        assert not self.markers
+
+
+def test_sole_test(django_pytester: DjangoPytester) -> None:
     """
     Make sure the database is configured when only Django TestCase classes
     are collected, without the django_db marker.
 
     Also ensures that the DB is available after a failure (#824).
     """
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         """
         import os
 
@@ -89,7 +126,7 @@ def test_sole_test(django_testdir) -> None:
     """
     )
 
-    result = django_testdir.runpytest_subprocess("-v")
+    result = django_pytester.runpytest_subprocess("-v")
     result.stdout.fnmatch_lines(
         [
             "*::test_foo FAILED",
@@ -106,8 +143,8 @@ def test_sole_test(django_testdir) -> None:
 class TestUnittestMethods:
     "Test that setup/teardown methods of unittests are being called."
 
-    def test_django(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_django(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
 
@@ -131,7 +168,7 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v", "-s")
+        result = django_pytester.runpytest_subprocess("-v", "-s")
         result.stdout.fnmatch_lines(
             [
                 "CALLED: setUpClass",
@@ -143,8 +180,8 @@ class TestUnittestMethods:
         )
         assert result.ret == 0
 
-    def test_setUpClass_not_being_a_classmethod(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_setUpClass_not_being_a_classmethod(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
 
@@ -157,7 +194,7 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v", "-s")
+        result = django_pytester.runpytest_subprocess("-v", "-s")
         expected_lines = [
             "* ERROR at setup of TestFoo.test_pass *",
             "E * TypeError: *",
@@ -165,8 +202,8 @@ class TestUnittestMethods:
         result.stdout.fnmatch_lines(expected_lines)
         assert result.ret == 1
 
-    def test_setUpClass_multiple_subclasses(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_setUpClass_multiple_subclasses(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
 
@@ -191,7 +228,7 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
+        result = django_pytester.runpytest_subprocess("-v")
         result.stdout.fnmatch_lines(
             [
                 "*TestFoo::test_shared PASSED*",
@@ -203,8 +240,8 @@ class TestUnittestMethods:
         )
         assert result.ret == 0
 
-    def test_setUpClass_mixin(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_setUpClass_mixin(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
 
@@ -225,14 +262,12 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
-        result.stdout.fnmatch_lines(
-            ["*TestFoo::test_foo PASSED*", "*TestBar::test_bar PASSED*"]
-        )
+        result = django_pytester.runpytest_subprocess("-v")
+        result.stdout.fnmatch_lines(["*TestFoo::test_foo PASSED*", "*TestBar::test_bar PASSED*"])
         assert result.ret == 0
 
-    def test_setUpClass_skip(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_setUpClass_skip(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
             import pytest
@@ -260,7 +295,7 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v")
+        result = django_pytester.runpytest_subprocess("-v")
         result.stdout.fnmatch_lines(
             [
                 "*TestFoo::test_shared SKIPPED*",
@@ -272,8 +307,8 @@ class TestUnittestMethods:
         )
         assert result.ret == 0
 
-    def test_multi_inheritance_setUpClass(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_multi_inheritance_setUpClass(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import TestCase
 
@@ -334,12 +369,12 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-vvvv", "-s")
+        result = django_pytester.runpytest_subprocess("-vvvv", "-s")
         assert result.parseoutcomes()["passed"] == 6
         assert result.ret == 0
 
-    def test_unittest(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_unittest(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from unittest import TestCase
 
@@ -363,7 +398,7 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-v", "-s")
+        result = django_pytester.runpytest_subprocess("-v", "-s")
         result.stdout.fnmatch_lines(
             [
                 "CALLED: setUpClass",
@@ -375,8 +410,8 @@ class TestUnittestMethods:
         )
         assert result.ret == 0
 
-    def test_setUpClass_leaf_but_not_in_dunder_dict(self, django_testdir) -> None:
-        django_testdir.create_test_module(
+    def test_setUpClass_leaf_but_not_in_dunder_dict(self, django_pytester: DjangoPytester) -> None:
+        django_pytester.create_test_module(
             """
             from django.test import testcases
 
@@ -397,10 +432,8 @@ class TestUnittestMethods:
         """
         )
 
-        result = django_testdir.runpytest_subprocess("-q", "-s")
-        result.stdout.fnmatch_lines(
-            ["*FooBarTestCase.setUpClass*", "*test_noop*", "1 passed*"]
-        )
+        result = django_pytester.runpytest_subprocess("-q", "-s")
+        result.stdout.fnmatch_lines(["*FooBarTestCase.setUpClass*", "*test_noop*", "1 passed*"])
         assert result.ret == 0
 
 
@@ -420,7 +453,7 @@ class TestCaseWithTrDbFixture(TestCase):
         assert 1
 
 
-def test_pdb_enabled(django_testdir) -> None:
+def test_pdb_enabled(django_pytester: DjangoPytester) -> None:
     """
     Make sure the database is flushed and tests are isolated when
     using the --pdb option.
@@ -429,7 +462,7 @@ def test_pdb_enabled(django_testdir) -> None:
     https://github.com/pytest-dev/pytest-django/issues/405
     """
 
-    django_testdir.create_test_module(
+    django_pytester.create_test_module(
         '''
         import os
 
@@ -461,12 +494,12 @@ def test_pdb_enabled(django_testdir) -> None:
     '''
     )
 
-    result = django_testdir.runpytest_subprocess("-v", "--pdb")
+    result = django_pytester.runpytest_subprocess("-v", "--pdb")
     assert result.ret == 0
 
 
-def test_debug_not_used(django_testdir) -> None:
-    django_testdir.create_test_module(
+def test_debug_not_used(django_pytester: DjangoPytester) -> None:
+    django_pytester.create_test_module(
         """
         from django.test import TestCase
 
@@ -483,6 +516,6 @@ def test_debug_not_used(django_testdir) -> None:
     """
     )
 
-    result = django_testdir.runpytest_subprocess("--pdb")
+    result = django_pytester.runpytest_subprocess("--pdb")
     result.stdout.fnmatch_lines(["*= 1 passed*"])
     assert result.ret == 0
