@@ -3,6 +3,8 @@ from django.conf import settings
 from django.urls import is_valid_path
 from django.utils.encoding import force_str
 
+from .helpers import DjangoPytester
+
 
 @pytest.mark.urls("pytest_django_test.urls_overridden")
 def test_urls() -> None:
@@ -16,8 +18,16 @@ def test_urls_client(client) -> None:
     assert force_str(response.content) == "Overridden urlconf works!"
 
 
-def test_urls_cache_is_cleared(testdir) -> None:
-    testdir.makepyfile(
+@pytest.mark.django_project(
+    extra_settings="""
+    ROOT_URLCONF = "empty"
+    """,
+)
+def test_urls_cache_is_cleared(django_pytester: DjangoPytester) -> None:
+    django_pytester.makepyfile(
+        empty="""
+        urlpatterns = []
+        """,
         myurls="""
         from django.urls import path
 
@@ -25,10 +35,10 @@ def test_urls_cache_is_cleared(testdir) -> None:
             pass
 
         urlpatterns = [path('first', fake_view, name='first')]
-    """
+        """,
     )
 
-    testdir.makepyfile(
+    django_pytester.create_test_module(
         """
         from django.urls import reverse, NoReverseMatch
         import pytest
@@ -37,20 +47,28 @@ def test_urls_cache_is_cleared(testdir) -> None:
         def test_something():
             reverse('first')
 
-
         def test_something_else():
             with pytest.raises(NoReverseMatch):
                 reverse('first')
-
-    """
+        """,
     )
 
-    result = testdir.runpytest_subprocess()
+    result = django_pytester.runpytest_subprocess()
     assert result.ret == 0
 
 
-def test_urls_cache_is_cleared_and_new_urls_can_be_assigned(testdir) -> None:
-    testdir.makepyfile(
+@pytest.mark.django_project(
+    extra_settings="""
+    ROOT_URLCONF = "empty"
+    """,
+)
+def test_urls_cache_is_cleared_and_new_urls_can_be_assigned(
+    django_pytester: DjangoPytester,
+) -> None:
+    django_pytester.makepyfile(
+        empty="""
+        urlpatterns = []
+        """,
         myurls="""
         from django.urls import path
 
@@ -58,10 +76,7 @@ def test_urls_cache_is_cleared_and_new_urls_can_be_assigned(testdir) -> None:
             pass
 
         urlpatterns = [path('first', fake_view, name='first')]
-    """
-    )
-
-    testdir.makepyfile(
+        """,
         myurls2="""
         from django.urls import path
 
@@ -69,10 +84,10 @@ def test_urls_cache_is_cleared_and_new_urls_can_be_assigned(testdir) -> None:
             pass
 
         urlpatterns = [path('second', fake_view, name='second')]
-    """
+        """,
     )
 
-    testdir.makepyfile(
+    django_pytester.create_test_module(
         """
         from django.urls import reverse, NoReverseMatch
         import pytest
@@ -87,8 +102,8 @@ def test_urls_cache_is_cleared_and_new_urls_can_be_assigned(testdir) -> None:
                 reverse('first')
 
             reverse('second')
-    """
+        """,
     )
 
-    result = testdir.runpytest_subprocess()
+    result = django_pytester.runpytest_subprocess()
     assert result.ret == 0
